package com.ruoyi.bst.bonus.service.impl;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.bst.balanceLog.domain.enums.BalanceLogBstType;
import com.ruoyi.bst.bonus.domain.Bonus;
import com.ruoyi.bst.bonus.domain.BonusQuery;
import com.ruoyi.bst.bonus.domain.BonusVO;
import com.ruoyi.bst.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.bst.bonus.domain.enums.BonusBstType;
import com.ruoyi.bst.bonus.domain.enums.BonusStatus;
import com.ruoyi.bst.bonus.mapper.BonusMapper;
import com.ruoyi.bst.bonus.service.BonusService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.user.service.UserService;

import lombok.extern.slf4j.Slf4j;

/**
 * 分成明细Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-26
 */
@Service
@Slf4j
public class BonusServiceImpl implements BonusService
{
    @Autowired
    private BonusMapper bonusMapper;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private UserService userService;

    /**
     * 查询分成明细
     *
     * @param id 分成明细主键
     * @return 分成明细
     */
    @Override
    public BonusVO selectBonusById(Long id)
    {
        return bonusMapper.selectBonusById(id);
    }

    /**
     * 查询分成明细列表
     *
     * @param bonus 分成明细
     * @return 分成明细
     */
    @Override
    public List<BonusVO> selectBonusList(BonusQuery bonus)
    {
        return bonusMapper.selectBonusList(bonus);
    }

    /**
     * 新增分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int insertBonus(Bonus bonus)
    {
        bonus.setCreateTime(DateUtils.getNowDate());
        return bonusMapper.insertBonus(bonus);
    }

    /**
     * 修改分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int updateBonus(Bonus bonus)
    {
        return bonusMapper.updateBonus(bonus);
    }

    /**
     * 批量删除分成明细
     *
     * @param ids 需要删除的分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusByIds(Long[] ids)
    {
        return bonusMapper.deleteBonusByIds(ids);
    }

    /**
     * 删除分成明细信息
     *
     * @param id 分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusById(Long id)
    {
        return bonusMapper.deleteBonusById(id);
    }

    @Override
    public int batchInsert(List<? extends Bonus> list) {
        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }
        Date now = DateUtils.getNowDate();
        list.forEach(bonus -> {
            bonus.setCreateTime(now);
        });
        return bonusMapper.batchInsert(list);
    }

    @Override
    public boolean prepayByBst(BonusBstType bstType, Long bstId) {
        if (bstType == null || bstId == null) {
            return false;
        }
        
        // 查询分成列表
        BonusQuery query = new BonusQuery();
        query.setBstType(bstType.getType());
        query.setBstId(bstId);
        List<BonusVO> list = this.selectBonusList(query);
        int rows = prepay(list);
        return rows == list.size();
    }

    // 预分成
    private int prepay(List<BonusVO> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }

        // 将未出账的金额转到待分成金额，并且保证金额不超总数
        // 后期如果有风控，需要每个分成单独处理
        List<Long> ids = list.stream().map(Bonus::getId).collect(Collectors.toList());
        int rows = bonusMapper.prepay(ids, LocalDateTime.now());
        
        // 如果预分成成功，则立即执行一次分成
        if (rows > 0) {
            scheduledExecutorService.schedule(() -> {
                this.payBonusBeforeTime(LocalDateTime.now());
            }, 3, TimeUnit.SECONDS);
        }

        return rows;
    }

    @Override
    public int payBonusBeforeTime(LocalDateTime time) {
        // 查询预计分成时间当前时间之前，且为待分成的分成单
        BonusQuery query = new BonusQuery();
        query.setStatus(BonusStatus.WAIT_DIVIDE.getStatus());
        query.setPrePayTimeEnd(time);
        List<BonusVO> list = this.selectBonusList(query);

        if (CollectionUtils.isEmptyElement(list)) {
            log.info("暂无待分成的分成单");
            return 0;
        }

        int sum = 0;
        for (BonusVO bonus : list) {
            try {
                sum += this.payBonus(bonus);
            } catch (Exception e) {
                log.warn("分成打款失败：id={}, e={}", bonus.getId(), e.getMessage());
            }
        }

        return sum;
    }

    @Override
    public int payBonus(BonusVO bonus) {
        if (bonus == null) {
            return 0;
        }
        Integer result = transactionTemplate.execute(status -> {
            // 更新分成状态为已分成
            int pay = bonusMapper.pay(bonus.getId(), bonus.getWaitAmount(), LocalDateTime.now());
            ServiceUtil.assertion(pay != 1, "分成打款失败，待分金额不足或状态已改变");

            // 若需要打款到余额，根据类型，添加金额
            if (bonus.getToBalance() != null && bonus.getToBalance()) {
                if (BonusArrivalType.userList().contains(bonus.getArrivalType())) {
                    BalanceLogBstType bstType = BalanceLogBstType.parseByBonusBstType(bonus.getBstType());
                    int add = userService.addBalance(bonus.getArrivalId(), bonus.getWaitAmount(), bstType, bonus.getBstId(), bonus.getReason());
                    ServiceUtil.assertion(add != 1, "为用户%s增加账户金额失败", bonus.getArrivalId());
                }
            }

            return pay;
        });

        return result == null ? 0 : result;
    }

}
